/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
using Gee;

public class Border : Pod {
	/**
	 * Each seed (defined in Pod) has a motif
	 */
	public ArrayList<Pod> motifs = new ArrayList<Pod> ();
	public Pod default_motif;

	public Border (Pod motif) {
		base ();
		default_motif = motif;
	}

	protected override string serialize_aux () {
		var s = "";
		// Serialize the motifs, each ONCE
		var unique_list = new ArrayList<Pod> ();
		unique_list.add (default_motif);
		s += "\nMotif\n" + default_motif.serialize_aux ();
		foreach (var motif in motifs) {
			if (Pod.search (unique_list, motif) == -1) {
				unique_list.add (motif);
				s += "\nMotif\n" + motif.serialize_aux ();
			}
		}
		// Serialize the border
		s += "\nBorder\n";
		foreach (var seed in this) s += seed.serialize (true);
		s += "\nMotifs";
		foreach (var motif in motifs) {
			s += " %d".printf (Pod.search (unique_list, motif));
		}
		return s;
	}

	public override void compile (string s) {
		// ss = { "the motifs", "the border" }
		var ss = s.split ("\nBorder\n");
		if (ss.length < 2) return;
		var unique_list = new ArrayList<Pod> ();
		// ss[0] = { "some prelude to ignore", "a motif", "another motif", ... }
		var ms = ss[0].split ("\nMotif\n");
		if (ms.length < 2) return;
		for (int i=1; i < ms.length; i++) {
			var motif = new Pod ();
			motif.compile (ms[i]);
			unique_list.add (motif);
		}
		default_motif = unique_list[0];
		// ss[1] = { "the border pod", "the indices to unique_list" }
		var bs = ss[1].split ("\nMotifs");
		if (bs.length < 2) return;
		base.compile (bs[0]);
		// bs[1] = {"index", "index", "index", ... }
		motifs.clear ();
		foreach (var idx in bs[1] .strip () .split (" ")) {
			var n = int.parse (idx);
			if (n < 0 || n >= unique_list.size) n = 0;
			motifs.add (unique_list[n]);
			if (motifs.size == this.size) return;
		}
		// Pad extra motifs if there aren't enough indices
		for (int i=motifs.size; i < this.size; i++) {
			motifs.add (unique_list[0]);
		}
	}

	public override Seed? add_seed_at_point (Type type, Point p) {
		assert (size == motifs.size);
		motifs.add (default_motif);
		return base.add_seed_at_point (type, p);
	}

	public Pod get_motif (int n) {
		return motifs[n];
	}

	public override void remove (Seed seed) {
		var n = index_of (seed);
		if (n >= 0) {
			base.remove (seed);
			motifs.remove_at (n);
		}
	}

	public override void add (Seed seed) {
		base.add (seed);
		motifs.add (default_motif);
	}

	public override void clear () {
		base.clear ();
		motifs.clear ();
	}

	/**
	 * Swap seed positions
	 */
	public override void swap (int i, int j) {
		base.swap (i, j);
		var t = motifs[i];
		motifs[i] = motifs[j];
		motifs[j] = t;
	}
}
// vim: tabstop=4: textwidth=100: autoindent:
