/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
public class ImageSizeDialog : Object {
	struct SizeSpec {
		public string text;
		public int width;
		public int height;
		public SizeSpec (string t, int w, int h) {
			text = t;
			width = w;
			height = h;
		}
	}

	/**
	 * Pre-set sizes
	 */
	SizeSpec[] size_specs;
	public ImageSizeDialog () {
		size_specs = {
			SizeSpec (_("640x480"), 640, 480),
			SizeSpec (_("800x600"), 800, 600),
			SizeSpec (_("1024x768"), 1024, 768),
			SizeSpec (_("1600x1200"), 1600, 1200),
			SizeSpec (_("A3 (300ppi)"), 3508, 4960),
			SizeSpec (_("A4 (300ppi)"), 2480, 3508),
			SizeSpec (_("A5 (300ppi)"), 1754, 2480),
			SizeSpec (_("A6 (300ppi)"), 1240,1754)
		};
	}

	/**
	 * Widgets to set by the run method
	 */
	Gtk.Dialog dialog = null;
	Gtk.SpinButton width_entry;
	Gtk.SpinButton height_entry;

	public void run (Gtk.Window parent_window, PodDrawing pod_drawing) {
		if (dialog == null) make_dialog (parent_window);
		width_entry.value = pod_drawing.pod_width;
		height_entry.value = pod_drawing.pod_height;
		var result = dialog.run ();
		dialog.hide ();
		if (result == Gtk.ResponseType.OK) {
			pod_drawing.resize ((int) width_entry.value, (int) height_entry.value);
		}
	}

	void make_dialog (Gtk.Window parent_window) {
		dialog = Util.make_dialog (parent_window, _("Image Size"), _("_Set Size"), (grid) => {
			// The user can select a size from a combobox
			var template = new Gtk.ComboBoxText ();
			foreach (var ss in size_specs) {
				template.append_text (ss.text);
			}
			var y = 0;
			grid.attach (Util.title (_("Template")), 0, y++, 2, 1);
			grid.attach (template, 0, y++, 2, 1);
			template.halign = Gtk.Align.FILL;
			template.hexpand = true;

			// The user can also enter custom size not available from the combobox
			width_entry = new Gtk.SpinButton.with_range (10, 10000, 10);
			height_entry = new Gtk.SpinButton.with_range (10, 10000, 10);
			grid.attach (Util.title (_("Image Size")), 0, y++, 2, 1);
			Util.attach (grid, y++, _("Width:"), width_entry);
			Util.attach (grid, y++, _("Height:"), height_entry);

			// Make the combobox consistent with the spinbuttons
			var ignore_signal = false;
			template.changed.connect (() => {
				var n = template.active;
				if (n == -1) return;
				ignore_signal = true;
				width_entry.value = size_specs[n].width;
				height_entry.value = size_specs[n].height;
				ignore_signal = false; });
			width_entry.value_changed.connect (() => {
				if (!ignore_signal) template.active = -1; });
			height_entry.value_changed.connect (() => {
				if (!ignore_signal) template.active = -1; });
		});//end make_dialog
	}
}
// vim: tabstop=4: textwidth=100: autoindent:
