/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
using Gee;

public class MotifButton : Gtk.Button, IRender {
	Pod motif;
	Gtk.DrawingArea icon;
	int size = 30;
	int gap_x = 10;
	int gap_y = 5;
	int width;
	int height;

	RenderJob render_job;

	public MotifButton (Pod motif) {
		this.motif = motif;
		this.image = icon = new Gtk.DrawingArea ();
		icon.set_has_window (false);
		icon.set_size_request (width = gap_x * 2 + size, height = gap_y * 2 + size);
		icon.draw.connect (draw_icon);
		motif.signal_changed.connect (render_image_surface);
	}

	bool draw_icon (Cairo.Context cr) {
		if ((get_state_flags () & Gtk.StateFlags.INSENSITIVE) != 0 || motif == null) return true;
		render_job.paint_image_surface (cr, 0, 0);
		return true;
	}

	/**
	 * IRender implementation
	 */
	public void render (double fraction) {
		icon.queue_draw ();
	}

	void render_image_surface () {
		if (motif == null) return;
		var line = new Line ();
		line.init_full ({gap_x, gap_y, gap_x + size, gap_y});
		line.base_offset = 0;
		line.width0 = line.width1 = size;
		line.num_segment = 1;
		line.flip_even = Flip.NONE;
		line.tweak = motif.tweak;
		line.level_of_detail = 100;
		var pod = new Border (motif);
		pod.add (line);
		render_job = RenderJob.create_named ("button", pod, 0, 0, width, height, this);
	}

}

// vim: tabstop=4: textwidth=100: autoindent:
