/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
using Gee;

public class MutiaraApplication : Gtk.Application {

	public static MutiaraApplication instance;

	public MutiaraApplication () {
		Object (application_id: "bn.mutiara", flags: ApplicationFlags.FLAGS_NONE);
	}

	protected override void activate () {
		var main_window = new Gtk.ApplicationWindow (this);
		try {
			main_window.set_icon_from_file (Config.PIXMAPSDIR + "/mutiara.png");
		}
		catch (Error e) {
			stderr.printf ("File error: %s\n", e.message);
		}

		var main_box = new Gtk.Grid ();
		main_window.add (main_box);

		var pod_editor = new PodEditor (main_window, create_border ());
		main_box.add (pod_editor);
		pod_editor.halign = Gtk.Align.FILL;
		pod_editor.valign = Gtk.Align.FILL;

		main_window.show_all ();
		main_window.resize (800, 800);
	}

	Border create_border () {
		string s = """
Motif
Arc (0.5, 0.5) (0.5, 0.3) (3.141, 6.282) 0
	segmentation 0.5, 9
	width 0.5, 0.5
	replication 0, 100, 0, 0
	color (0.9, 0.3, 0.3, 1) 0.5
	style 0
Arc (0.5, 0.75) (0.1, 0.1) (0, 6.282) 0
	segmentation 0.5, 10
	width 0.2, 0.2
	replication 0, 100, 0, 1
	color (0.2, 0.2, 0.9, 1) 0.5
	style 0

Border
Motifs
""";
		var border = new Border (new Pod ());
		border.compile (s);
		return border;
	}

	public static int main (string[] args) {
		if (!Thread.supported()) {
			stderr.printf ("Cannot run without threads.\n");
			return 1;
		}

		// Localisation stuff
		Intl.bindtextdomain (Config.GETTEXT_PACKAGE, Config.LOCALEDIR);
		Intl.bind_textdomain_codeset (Config.GETTEXT_PACKAGE, "UTF-8");
		Intl.textdomain (Config.GETTEXT_PACKAGE);

		// Register new Glib types
		typeof (Arc) .name ();
		typeof (Line) .name ();
		typeof (Curve) .name ();

		instance = new MutiaraApplication ();
		return instance.run (args);
	}
}
// vim: tabstop=4: textwidth=100: autoindent:
