/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Static methods that are generally useful
 */

namespace Util {
	bool debug_on () {
		return Environment.get_variable ("MUTIARA_DEBUG") != null;
	}

	/**
	 * Remove trailing zeros and decimal point.
	 * e.g., 2.000 => 2
	 *       5.240 => 5.24
	 */
	string format (double n, int decimal_places) {
		var format_string = "%%.%df".printf (decimal_places);
		var number = format_string.printf (n);
		try {
			number = /\.?0+$/.replace (number, -1, 0, "");
			if (number == "-0") return "0";
			else return number;
		}
		catch (RegexError e) {
			GLib.print ("%s\n", e.message);
			return number;
		}
	}

	/**
	 * Create left-justified, left-positioned label
	 */
	Gtk.Label label (string label_text) {
		var label = new Gtk.Label (label_text);
		label.halign = Gtk.Align.START;
		label.justify = Gtk.Justification.LEFT;
		return label;
	}

	/**
	 * Create title label
	 */
	Gtk.Label title (string label_text) {
		var label = Util.label_centered ("<b>" + label_text + "</b>");
		label.use_markup = true;
		return label;
	}

	/**
	 * Create centered label
	 */
	Gtk.Label label_centered (string label_text) {
		var label = new Gtk.Label (label_text);
		label.halign = Gtk.Align.CENTER;
		label.justify = Gtk.Justification.CENTER;
		label.margin_top = 10;
		label.margin_bottom = 10;
		return label;
	}

	/**
	 * Attach a widget to a table complete with a left-justified label
	 */
	void attach (Gtk.Grid grid, int y, string? label_text, Gtk.Widget? widget) {
		if (label_text != null) grid.attach (Util.label (label_text), 0, y, 1, 1);
		grid.attach (widget, 1, y, 1, 1);
	}

	/**
	 * Create a modal, non-resizable dialog box.
	 */
	delegate void ContentsDelegate (Gtk.Grid grid);

	Gtk.Dialog make_dialog (Gtk.Window parent, string title, string ok_text, ContentsDelegate put_contents) {
		var dialog = new Gtk.Dialog.with_buttons (title, parent, Gtk.DialogFlags.MODAL,
			_("Cancel"), Gtk.ResponseType.CANCEL, ok_text, Gtk.ResponseType.OK);
		dialog.resizable = false;
		var box = dialog.get_content_area ();
		var grid = new Gtk.Grid ();
		box.add (grid);
		//box.spacing = 20;
		grid.row_spacing = 5;
		grid.column_spacing = 10;
		grid.margin_left = 5;
		grid.margin_right = 5;
		grid.margin_bottom = 20;
		put_contents (grid);
		dialog.show_all ();
		return dialog;
	}

	/**
	 * Call this method with text editing components
	 */
	void resolve_accelerator_clashes (Gtk.Widget widget) {
		widget.focus_in_event.connect ((e) => {
			MutiaraApplication.instance.remove_accelerator ("win.Delete", null);
			return false;
		});
		widget.focus_out_event.connect ((e) => {
			MutiaraApplication.instance.add_accelerator ("Delete", "win.Delete", null);
			return false;
		});
	}

	/**
	 * Create a spin button which disable some accelerators upon focus.
	 */
	Gtk.SpinButton make_spin_button (double min, double max, double step) {
		var button = new Gtk.SpinButton.with_range (min, max, step);
		resolve_accelerator_clashes (button);
		return button;
	}
}

// vim: tabstop=4: textwidth=100: autoindent:
