/*  Mutiara motif designer.
	Copyright (C) 2011  Nor Jaidi Tuah

	This file is part of Mutiara.

	Mutiara is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
public class Zoom : Object {
	public signal void signal_changed (double z);

	double left = 10;
	double top = 10;
	double width = 20;
	double length = 200;
	double radius = 8;
	double thumb = 0.5;  // 0 to 1

	bool dragging = false;
	double drag_x;

	public void draw (Cairo.Context cr, double background) {
		cr.set_source_rgba (background, background, background, 0.5);
		cr.rectangle (left - 5, top - 5, length + 10, width + 10);
		cr.fill ();
		// Show ticks
		var color = 1 - background;
		cr.set_source_rgba (color, color, color, 0.2);
		cr.set_line_width (1.0);
		cr.rectangle (left, top, 1, width);
		cr.rectangle (left + length / 2 - 0.5, top, 1, width);
		cr.rectangle (left + length - 1, top, 1, width);
		cr.rectangle (left, top + width / 2 - 0.5, length, 1);
		cr.stroke ();
		// Show thumb
		var x = left + thumb * length;
		var y = top + width / 2;
		var p = new Cairo.Pattern.radial (x + 0.5 * radius, y + 0.5 * radius, 0, x, y, radius);
		p.add_color_stop_rgba (0, 1, 0, 0, 0.6);
		p.add_color_stop_rgba (0.7, 1, 0, 0, 0.6);
		p.add_color_stop_rgba (1, 1, 0, 0, 0);
		cr.set_source (p);
		cr.arc (x, y, radius, 0, 2 * Math.PI);
		cr.fill ();
	}

	public bool press (double x, double y) {
		var thumb_x = left + thumb * length;
		var thumb_y = top + width / 2;
		drag_x = x - thumb_x;
		return dragging = Math.hypot (thumb_x - x, thumb_y - y) <= radius;
	}

	public bool move (double x, double y) {
		if (!dragging) return false;
		var mid_x = left + 0.5 * length;
		var thumb_x = x - drag_x;
		// Snap thumb_x to mid_x
		if (Math.fabs (thumb_x - mid_x) <= radius) thumb_x = mid_x;
		thumb = ((thumb_x - left) / length).clamp (0, 1);
		change ();
		return true;
	}

	public bool release (double x, double y) {
		if (!dragging) return false;
		dragging = false;
		return true;
	}

	/**
	 * Convert thumb (0 to 1) to scale (0.05 to 4)
	 */
	void change () {
		// (0 - 0.5) to (0.05 - 1)
		if (thumb <= 0.5) {
			signal_changed (0.05 + (1 - 0.05) * thumb * 2);
		}
		// (0.5 - 1) to (1 - 4)
		else {
			signal_changed (1 + (4 - 1) * (thumb - 0.5) * 2);
		}
	}
}
// vim: tabstop=4: textwidth=100: autoindent:
